<?php

namespace Drupal\jsnlog\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Asset\LibraryDiscovery;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure maintenance settings for this site.
 *
 * @internal
 */
class JsnlogSettingsForm extends ConfigFormBase {

  public const LIBRARY_DIR = 'libraries/jsnlog';

  /**
   * The state keyvalue collection.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The permission handler.
   *
   * @var \Drupal\user\PermissionHandlerInterface
   */
  protected $permissionHandler;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * The library discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscovery
   */
  private $libraryDiscovery;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  private $fileSystem;

  /**
   * Constructs a new JSNLogSettingsForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state key value collection to use.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger service.
   * @param \Drupal\Core\Asset\LibraryDiscovery $library_discovery
   *   The library discovery service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    StateInterface $state,
    Messenger $messenger,
    LibraryDiscovery $library_discovery,
    FileSystemInterface $file_system
  ) {
    parent::__construct($config_factory);
    $this->state = $state;
    $this->messenger = $messenger;
    $this->libraryDiscovery = $library_discovery;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('messenger'),
      $container->get('library.discovery'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'jsnlog_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['jsnlog.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    if (!$this->checkLibraryFolder()) {
      $this->messenger->addWarning($this->t('JSNLog javascript library not present under libraries. Download the library or use the cdn version.'));
    }

    $config = $this->config('jsnlog.settings');

    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('JSNLog settings'),
      '#open' => TRUE,
    ];

    $form['general']['enabled'] = [
      '#type' => 'radios',
      '#options' => ['1' => $this->t('enabled'), '0' => $this->t('disabled')],
      '#title' => $this->t('Enable JSNLog'),
      '#default_value' => $config->get('enabled'),
      '#description' => $this->t('Enable or disable javascript logging.'),
    ];

    $form['general']['logging_type'] = [
      '#type' => 'radios',
      '#options' => $this->getLogTypeOptions(),
      '#title' => $this->t('Logging Type'),
      '#default_value' => $config->get('logging_type'),
      '#description' => $this->t('Should be console and ajax errors be logged or only one of them.'),
    ];

    $form['general']['library_type'] = [
      '#type' => 'radios',
      '#options' => $this->getLibraryTypeOptions(),
      '#title' => $this->t('Library Type'),
      '#default_value' => $config->get('library_type'),
      '#description' => $this->t('Which library type should be loaded.'),
    ];

    $form['general']['debug_level'] = [
      '#type' => 'select',
      '#options' => $this->getDebugLevelOptions(),
      '#title' => $this->t('Debug Level'),
      '#default_value' => $config->get('debug_level'),
      '#description' => $this->t('Define the debug level of logging javascript messages.'),
    ];

    $form['general']['user_agent_regex'] = [
      '#title' => $this->t('User Agent Filter'),
      '#type' => 'textfield',
      '#default_value' => $config->get('user_agent_regex'),
      '#description' => $this->t('If defined, log messages only get processed if this regular expression matches the user agent string of the browser.'),
      '#maxlength' => 500,
    ];

    // C&P this part from matomo module. Big thx to them.
    $visibility_request_path_pages = $config->get('visibility.request_path_pages');

    $form['page_visibility_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Pages'),
      '#group' => 'page_scope',
    ];

    if ($config->get('visibility.request_path_mode') == 2 && !$php_access) {
      // No permission to change PHP snippets, but keep existing settings.
      $form['page_visibility_settings'] = [];
      $form['page_visibility_settings']['visibility_request_path_mode'] = ['#type' => 'value', '#value' => 2];
      $form['page_visibility_settings']['visibility_request_path_pages'] = ['#type' => 'value', '#value' => $visibility_request_path_pages];
    }
    else {
      $options = [
        $this->t('Every page except the listed pages'),
        $this->t('The listed pages only'),
      ];
      $description = $this->t("Specify pages by using their paths. Enter one path per line. 
        The '*' character is a wildcard. Example paths are %blog for the blog page and 
        %blog-wildcard for every personal blog. %front is the front page.",
        [
          '%blog' => '/blog',
          '%blog-wildcard' => '/blog/*',
          '%front' => '<front>',
        ]
      );

      $title = $this->t('Pages');

      $form['page_visibility_settings']['visibility_request_path_mode'] = [
        '#type' => 'radios',
        '#title' => $this->t('Add logging to specific pages'),
        '#options' => $options,
        '#default_value' => $config->get('visibility.request_path_mode'),
      ];
      $form['page_visibility_settings']['visibility_request_path_pages'] = [
        '#type' => 'textarea',
        '#title' => $title,
        '#title_display' => 'invisible',
        '#default_value' => !empty($visibility_request_path_pages) ? $visibility_request_path_pages : '',
        '#description' => $description,
        '#rows' => 10,
      ];
    }

    // Render the role overview.
    $visibility_user_role_roles = $config->get('visibility.user_role_roles');

    $form['role_visibility_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Roles'),
      '#group' => 'page_scope',
    ];

    $form['role_visibility_settings']['visibility_user_role_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Add logging for specific roles'),
      '#options' => [
        $this->t('Add to the selected roles only'),
        $this->t('Add to every role except the selected ones'),
      ],
      '#default_value' => $config->get('visibility.user_role_mode'),
    ];
    $form['role_visibility_settings']['visibility_user_role_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#default_value' => !empty($visibility_user_role_roles) ? $visibility_user_role_roles : [],
      '#options' => array_map('\Drupal\Component\Utility\Html::escape', user_role_names()),
      '#description' => $this->t('If none of the roles are selected, all users javascript errors will be logged. If a user has any of the roles checked, that user will be tracked (or excluded, depending on the setting above).'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Verify that every path is prefixed with a slash, but don't check PHP
    // code snippets and do not check for slashes if no paths configured.
    if ($form_state->getValue('visibility_request_path_mode') != 2 && !empty($form_state->getValue('visibility_request_path_pages'))) {
      $pages = preg_split('/(\r\n?|\n)/', $form_state->getValue('visibility_request_path_pages'));
      foreach ($pages as $page) {
        if (strpos($page, '/') !== 0 && $page !== '<front>') {
          $form_state->setErrorByName('visibility_request_path_pages', $this->t('Path "@page" not prefixed with slash.', ['@page' => $page]));
          // Drupal forms show one error only.
          break;
        }
      }
    }

    $form_state->setValue('visibility_request_path_pages', trim($form_state->getValue('visibility_request_path_pages')));
    $form_state->setValue('visibility_user_role_roles', array_filter($form_state->getValue('visibility_user_role_roles')));

    // Check Library type and validate.
    if ('ext' != $form_state->getValue('library_type')) {
      if (!$this->checkLibraryFolder()) {
        $form_state->setErrorByName('library_type', $this->t('JSNLog library is not present. Download the library or choose library type external.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('jsnlog.settings');
    $has_library_type_value_changed = ($config->get('library_type') != $form_state->getValue('library_type'));

    $config
      ->set('enabled', $form_state->getValue('enabled'))
      ->set('logging_type', $form_state->getValue('logging_type'))
      ->set('library_type', $form_state->getValue('library_type'))
      ->set('debug_level', $form_state->getValue('debug_level'))
      ->set('user_agent_regex', $form_state->getValue('user_agent_regex'))
      ->set('visibility.request_path_mode', $form_state->getValue('visibility_request_path_mode'))
      ->set('visibility.request_path_pages', $form_state->getValue('visibility_request_path_pages'))
      ->set('visibility.user_role_mode', $form_state->getValue('visibility_user_role_mode'))
      ->set('visibility.user_role_roles', $form_state->getValue('visibility_user_role_roles'))
      ->save();

    // Reset Library Cache if value changed.
    if ($has_library_type_value_changed) {
      $this->libraryDiscovery->clearCachedDefinitions();
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * Returns list of jsnlog appender options.
   *
   * @return array
   *   The appender option list.
   */
  private function getLogTypeOptions():array {
    return [
      'all' => $this->t('all'),
      'console' => $this->t('Console Logging'),
      'ajax' => $this->t('Ajax Logging'),
    ];
  }

  /**
   * Returns list of debug level options.
   *
   * @return array
   *   The debug level option list.
   */
  private function getDebugLevelOptions():array {
    return [
      1000 => $this->t('trace'),
      2000 => $this->t('debug'),
      3000 => $this->t('info'),
      4000 => $this->t('warn'),
      5000 => $this->t('error'),
      6000 => $this->t('fatal'),
    ];
  }

  /**
   * Returns a list of types to load js library.
   *
   * @return array
   *   The type option list.
   */
  private function getLibraryTypeOptions():array {
    return [
      '' => $this->t('default'),
      'min' => $this->t('minified'),
      'ext' => $this->t('external'),
    ];
  }

  /**
   * Check if JSNLog library is present.
   *
   * @return bool
   *   Whether or not library is present.
   */
  private function checkLibraryFolder():bool {
    $dir = static::LIBRARY_DIR;
    if ($this->fileSystem->prepareDirectory($dir)) {
      if (file_exists($dir . '/jsnlog.js') && file_exists($dir . '/jsnlog.min.js')) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
